@@ -488,14 +488,15 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
 	    entry = alloc_entry(1);
 	    entry->flags = ATF_PUBL;
 	}
-    if (entry->addr && (entry->flags & ATF_PERM) &&
+    if (!atmsvc_addr_in_use(*addr)) addr = NULL;
+    if (entry->addr && addr && (entry->flags & ATF_PERM) &&
       !atm_equal(entry->addr,addr,0,0)) {
 	diag(COMPONENT,DIAG_ERROR,"ignoring attempt to change ATM address of "
 	  "permanent entry");
 	return;
     }
-    if (entry->state == as_valid && entry->ip == ip && entry->addr &&
-      atm_equal(entry->addr,addr,0,0)) return; /* no news */
+    if (entry->state == as_valid && entry->ip == ip && (!addr || (entry->addr
+      && atm_equal(entry->addr,addr,0,0)))) return; /* no news */
     STOP_TIMER(entry);
     entry->ip = ip;
     if (!entry->itf) {
@@ -512,17 +513,19 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
     if (entry->itf != itf)
 	diag(COMPONENT,DIAG_ERROR,"interesting, interface has changed ... "
 	  "(%d -> %d)",entry->itf->number,itf->number);
-    if (!entry->addr) entry->addr = alloc(sizeof(*addr));
-    *entry->addr = *addr;
-    if (merge) {
-	ENTRY *incoming;
+    if (addr) {
+	if (!entry->addr) entry->addr = alloc(sizeof(*addr));
+	*entry->addr = *addr;
+	if (merge) {
+	    ENTRY *incoming;
 
-	while ((incoming = lookup_incoming(addr))) {
-	    STOP_TIMER(incoming);
-	    Q_REMOVE(unknown_incoming,incoming);
-	    incoming->vccs->entry = entry;
-	    Q_INSERT_HEAD(entry->vccs,incoming->vccs);
-	    free(incoming);
+	    while ((incoming = lookup_incoming(addr))) {
+		STOP_TIMER(incoming);
+		Q_REMOVE(unknown_incoming,incoming);
+		incoming->vccs->entry = entry;
+		Q_INSERT_HEAD(entry->vccs,incoming->vccs);
+		free(incoming);
+	    }
 	}
     }
     for (walk = entry->vccs; walk; walk = walk->next)
