@@ -17,6 +17,7 @@
 #include <linux/atm.h>
 #include <linux/atmclip.h> /* for CLIP_DEFAULT_IDLETIMER */
 #include <linux/atmarp.h>
+#include <linux/if_arp.h>
 
 #include "atm.h"
 #include "atmd.h"
@@ -43,7 +44,7 @@ void open_all(void)
 {
     struct sockaddr_atmsvc addr;
     struct atm_qos qos;
-    struct atm_blli blli;
+    struct atm_sap sap;
 
     if ((kernel = socket(PF_ATMSVC,SOCK_DGRAM,0)) < 0)
 	diag(COMPONENT,DIAG_FATAL,"socket: %s",strerror(errno));
@@ -56,13 +57,13 @@ void open_all(void)
     qos.rxtp.traffic_class = qos.txtp.traffic_class = ATM_ANYCLASS;
     if (setsockopt(incoming,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0)
 	diag(COMPONENT,DIAG_FATAL,"setsockopt SO_ATMQOS: %s",strerror(errno));
+    memset(&sap,0,sizeof(sap));
+    sap.blli[0].l2_proto = ATM_L2_ISO8802;
+    sap.blli[0].l3_proto = ATM_L3_NONE;
+    if (setsockopt(incoming,SOL_ATM,SO_ATMSAP,&sap,sizeof(sap)) < 0)
+	diag(COMPONENT,DIAG_FATAL,"setsockopt SO_ATMSAP: %s",strerror(errno));
     memset(&addr,0,sizeof(addr));
     addr.sas_family = AF_ATMSVC;
-    addr.sas_addr.bhli.hl_type = ATM_HL_NONE;
-    addr.sas_addr.blli = &blli;
-    blli.l2_proto = ATM_L2_ISO8802;
-    blli.l3_proto = ATM_L3_NONE;
-    blli.next = NULL;
     if (bind(incoming,(struct sockaddr *) &addr,sizeof(addr)) >= 0) {
 	if (listen(incoming,5) < 0)
 	    diag(COMPONENT,DIAG_FATAL,"listen: %s",strerror(errno));
@@ -239,6 +240,7 @@ static void accept_new(void)
     entry->state = as_invalid;
     entry->addr = alloc_t(struct sockaddr_atmsvc);
     *entry->addr = addr;
+    entry->flags = ATF_PUBL;
     Q_INSERT_HEAD(unknown_incoming,entry);
     vcc->entry = entry;
     Q_INSERT_HEAD(entry->vccs,vcc);
@@ -255,22 +257,6 @@ int connect_vcc(struct sockaddr *remote,const struct atm_qos *qos,int timeout)
 	diag(COMPONENT,DIAG_ERROR,"socket: %s",strerror(errno));
 	return -errno;
     }
-#if 0 /* ilmid and sigd do that now for us */
-    if (local) {
-    if (debug) {
-	int i;
-
-	for (i = 0; i < sizeof(struct sockaddr_atmsvc); i++)
-	    printf("%02X ",((unsigned char *) local)[i]);
-	printf("\n");
-    }
-   ((struct sockaddr_atmsvc *) local)->sas_addr.bhli.hl_type = 1; /* uuuh @@@ */
-    if (bind(fd,local,sizeof(struct sockaddr_atmsvc)) < 0) {
-	diag(COMPONENT,DIAG_ERROR,"bind: %s",strerror(errno));
-	return -errno;
-    }
-}
-#endif
     if (setsockopt(fd,SOL_ATM,SO_ATMQOS,qos,sizeof(*qos)) < 0) {
 	diag(COMPONENT,DIAG_ERROR,"setsockopt SO_ATMQOS: %s",strerror(errno));
 	return -errno;
@@ -285,12 +271,16 @@ int connect_vcc(struct sockaddr *remote,const struct atm_qos *qos,int timeout)
 	return -errno;
     }
     if (remote->sa_family == AF_ATMSVC) { /* @@@ that's cheating */
-	static struct atm_blli blli;
+	struct atm_sap sap;
 
-	((struct sockaddr_atmsvc *) remote)->sas_addr.blli = &blli;
-	blli.l2_proto = ATM_L2_ISO8802;
-	blli.l3_proto = ATM_L3_NONE;
-	blli.next = NULL;
+	memset(&sap,0,sizeof(sap));
+	sap.blli[0].l2_proto = ATM_L2_ISO8802;
+	sap.blli[0].l3_proto = ATM_L3_NONE;
+	if (setsockopt(fd,SOL_ATM,SO_ATMSAP,&sap,sizeof(sap)) < 0) {
+	    diag(COMPONENT,DIAG_ERROR,"setsockopt SO_ATMSAP: %s",
+	      strerror(errno));
+	    return -errno;
+	}
     }
     /* PVC connect never blocks */
     if (connect(fd,remote,remote->sa_family == AF_ATMPVC ?
@@ -382,7 +372,18 @@ void poll_loop(void)
 	    if (entry->vccs->fd >= fds) fds = entry->vccs->fd+1;
 	}
 	ret = select(fds,&rset,&cset,NULL,next_timer());
-fprintf(stderr,"select = %d\n",ret);
+/*
+ * Now here's something strange: < 0.32 needed the exception mask to be NULL
+ * in order to work, due to a bug in atm_select. In 0.32, this has been fixed.
+ * Also, 2.1 kernels use the poll mechanism and not select, so select is
+ * emulated on top of poll. Now the funny bit is that, as soon as the exception
+ * set is non-NULL, when a non-blocking connect finishes, select returns one
+ * but has none if the possible bits set in either rset or cset. To make things
+ * even stranger, no exception is actually found in sys_select, so this must be
+ * some very odd side-effect ... The work-around for now is to simply pass NULL
+ * for the exception mask (which is the right thing to do anyway, but it'd be
+ * nice if doing a perfectly valid variation wouldn't blow up the system ...)
+ */
 #if 0
 {
   int i;
