@@ -115,8 +115,12 @@ static void dump_vcc(VCC *vcc)
 
 static void dump_entries(ENTRY *list)
 {
-    static const char *flag_name[] = { "???","com","PERM","PUBL","trailers",
-      "netmask","NULL","ARPSRV" }; /* lower case flags are not used by ATMARP */
+    static const char *flag_name[] = {
+	"???",	"com", "PERM", "PUBL",			/* 0x0001-0x0008 */
+	"trailers", "netmask", "dontpub", "magic",	/* 0x0010-0x0080 */
+	"???", "???", "???", "???",			/* 0x0100-0x0800 */
+	"NULL", "ARPSRV", "DEFQOS" };			/* 0x1000-0x8000 */
+    /* lower case flags are not used by ATMARP */
     ENTRY *entry;
     VCC *vcc;
     char addr_buf[MAX_ATM_ADDR_LEN+1];
@@ -132,7 +136,7 @@ static void dump_entries(ENTRY *list)
 		strcpy(addr_buf,"<error>");
 	ipp = (unsigned char *) &entry->ip;
 	*tmp = 0;
-	for (i = 0; i < 8; i++)
+	for (i = 0; i < 16; i++)
 	    if (entry->flags & (1 << i)) {
 		if (*tmp) strcat(tmp,",");
 		strcat(tmp,flag_name[i]);
