@@ -1,6 +1,6 @@
 /* table.h - ATMARP table */
  
-/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
  
 
 #ifndef TABLE_H
@@ -70,13 +70,14 @@ extern const char *vcc_state_name[];
 extern ITF *itfs;
 extern ENTRY *unknown_incoming;
 
-extern int pretty;
+extern int pretty,merge;
 
 
 ENTRY *alloc_entry(int svc);
 
 ENTRY *lookup_ip(const ITF *itf,unsigned long ip);
 ENTRY *lookup_addr(const ITF *itf,const struct sockaddr_atmsvc *addr);
+ENTRY *lookup_incoming(const struct sockaddr_atmsvc *addr);
 
 void dump_all(void);
 
