@@ -85,8 +85,6 @@ void update_nsap(int itf, AsnOid *netprefix, AsnOid *esi)
   int fd, m, n;
 
   addr.sas_family = AF_ATMSVC;
-  addr.sas_addr.bhli.hl_type = ATM_HL_NONE;
-  addr.sas_addr.blli = NULL;
   addr.sas_addr.pub[0] = 0;
   
   /* Convert net prefix BER to hex */
