@@ -21,6 +21,7 @@
 #include <linux/atmclip.h>
 #include <linux/netdevice.h> /* for MAX_ADDR_LEN for if_arp.h */
 #include <linux/if_arp.h> /* for ATF_PERM and ATF_PUBL */
+#include <linux/version.h>
 
 #include "atm.h"
 
@@ -194,7 +195,9 @@ int main(int argc,char **argv)
 	perror("socket");
 	return 1;
     }
+#if LINUX_VERSION_CODE >= 0x20100
     *req.arp_dev = 0;
+#endif
     if (ioctl(s,op,&req) < 0) {
 	perror("ioctl SIOCxARP");
 	return 1;
