@@ -289,7 +289,8 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
   
   /* We don't create connection to entity where we already have
      a connection. */
-  if (conn_already_exists(p_conn_info->addr.sas_addr.prv, NULL))
+  if (conn_already_exists(p_conn_info->addr.sas_addr.prv, NULL) &&
+      is_data_direct(p_conn_info))
     return STATUS_K_ALREADY_OPEN;
 
   switch(p_conn_info->blli.l3.tr9577.snap[4]) { /* Kludge.  Eh? */
