@@ -60,7 +60,6 @@ kernel_init(unsigned char *mac_addr, int itf_num)
 {
   int rvalue;
   struct atmlec_msg msg;
-  struct atmif_sioc req;
 
   lec_socket = socket(PF_ATMSVC, SOCK_DGRAM, 0);
   if (lec_socket <0) {
@@ -73,20 +72,7 @@ kernel_init(unsigned char *mac_addr, int itf_num)
     close(lec_socket);
     return -1;
   }
-  if (!mac_addr) {
-    /* If mac address is not given, it is set from ESI */
-    req.number = 0;
-    req.length = ESI_LEN;
-    req.arg = &msg.content.normal.mac_addr;
-    
-    rvalue = ioctl(lec_socket, ATM_GETESI, &req);
-    if (rvalue <0) {
-      EVENT(EM_SERR,("Can't get ESI:%s\n",strerror(errno)));
-      close(lec_socket);
-      return -1;
-    }
-  } else
-    memcpy(&msg.content.normal.mac_addr, mac_addr, ETH_ALEN);
+  memcpy(&msg.content.normal.mac_addr, mac_addr, ETH_ALEN);
 
   msg.type = l_set_mac_addr;
   
