@@ -1,6 +1,6 @@
 /* arequipa.h - AREQUIPA support functions */
  
-/* Written 1996 by Jean-Michel Pittet and Werner Almesberger, EPFL-LRC */
+/* Written 1996,1997 by Jean-Michel Pittet and Werner Almesberger, EPFL-LRC */
  
 
 #ifndef AREQUIPA_H
@@ -11,6 +11,7 @@
 int arequipa_preset(int sd,const struct sockaddr_atmsvc *addr,
   const struct atm_qos *qos);
 int arequipa_expect(int sd,int on);
+int arequipa_renegotiate(int sd,const struct atm_qos *qos);
 int arequipa_close(int sd);
 
 #endif
