@@ -1,6 +1,6 @@
 /* atmd.h - Functions useful for demons */
  
-/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
  
 
 #ifndef ATMD_H
@@ -62,6 +62,8 @@ int get_verbosity(const char *component);
 
 void vdiag(const char *component,int severity,const char *fmt,va_list ap);
 void diag(const char *component,int severity,const char *fmt,...);
+void diag_dump(const char *component,int severity,const char *title,
+  const unsigned char *data,int len);
 
 
 /*------------------------------ Timer support ------------------------------*/
@@ -79,6 +81,7 @@ typedef struct _timer {
 
 TIMER *start_timer(long usec,void (*callback)(void *user),void *user);
 void stop_timer(TIMER *timer);
+void (*timer_handler(TIMER *timer))(void *user);
 struct timeval *next_timer(void);
 void pop_timer(TIMER *timer);
 void expire_timers(void);
