@@ -14,34 +14,12 @@
 #include "atm.h"
 
 
+#define fetch __atmlib_fetch
+
+
 #define RATE_ERROR -2
 
 
-static int fetch(const char **pos,...)
-{
-    const char *value;
-    int ref_len,best_len,len;
-    int i,best;
-    va_list ap;
-
-    va_start(ap,pos);
-    ref_len = strlen(*pos);
-    best_len = 0;
-    best = -1;
-    for (i = 0; (value = va_arg(ap,const char *)); i++) {
-	len = strlen(value);
-	if (*value != '!' && len <= ref_len && len > best_len &&
-	  !strncasecmp(*pos,value,len)) {
-	    best = i;
-	    best_len = len;
-	}
-    }
-    va_end(ap);
-    if (best > -1) (*pos) += best_len;
-    return best;
-}
-
-
 static int get_rate(const char **text,int up)
 {
     const char mult[] = "kKmMgGg";
@@ -109,13 +87,18 @@ static int params(const char **text,struct atm_trafprm *a,
 	switch (fetch(text,"max_pcr=","pcr=","min_pcr=","max_sdu=","sdu=",
 	  NULL)) {
 	    case 0:
-	    case 1:
 		if ((value = get_rate(text,0)) == RATE_ERROR) return -1;
 		if (a) a->max_pcr = value;
 		if (b) b->max_pcr = value;
 		break;
+	    case 1:
+		if ((value = get_rate(text,0)) == RATE_ERROR) return -1;
+		if (a) a->pcr = value;
+		if (b) b->pcr = value;
+		break;
 	    case 2:
 		if ((value = get_rate(text,1)) == RATE_ERROR) return -1;
+		if (value == ATM_MAX_PCR) return -1;
 		if (a) a->min_pcr = value;
 		if (b) b->min_pcr = value;
 		break;
