@@ -15,7 +15,18 @@
 #include <arpa/inet.h>
 
 #include <linux/atm.h>
+#include <linux/version.h>
+#if LINUX_VERSION_CODE >= 0x20100
 #include <linux/atm_tcp.h>
+#else
+
+struct atmtcp_hdr {
+    unsigned short vpi;
+    unsigned short vci;
+    unsigned short length; /* ... of data part */
+};
+
+#endif
 
 
 #define PORT 8402	/* old ATMTCP used 8401, but now the header is in
@@ -111,24 +122,32 @@ static void usage(const char *name)
 int main(int argc,char **argv)
 {
     struct sockaddr_in addr;
-    int listen_mode,port,itf;
+    const char *name;
+    int background,listen_mode,port,itf;
     int s2;
 
-    if (argc < 2) usage(argv[0]);
+    name = argv[0];
+    background = 0;
+    if (argc > 2 && !strcmp(argv[1],"-b")) {
+	background = 1;
+	argc--;
+	argv++;
+    }
+    if (argc < 2) usage(name);
     listen_mode = !strcmp(argv[1],"-l");
     if (listen_mode) {
-	if (argc > 3) usage(argv[0]);
+	if (argc > 3) usage(name);
 	port = argc == 3 ? atoi(argv[2]) : -1;
     }
     else {
-	if (argc > 4 || strcmp(argv[1],"-c")) usage(argv[0]);
+	if (argc > 4 || strcmp(argv[1],"-c")) usage(name);
 	port = argc == 4 ? atoi(argv[3]) : -1;
     }
     if ((s_tcp = socket(PF_INET,SOCK_STREAM,0)) < 0) {
 	perror("socket");
 	return 1;
     }
-    if (!port) usage(argv[0]);
+    if (!port) usage(name);
     addr.sin_family = AF_INET;
     addr.sin_port = htons(port == -1 ?  PORT :  port);
     /*
@@ -178,6 +197,16 @@ int main(int argc,char **argv)
 	}
     }
     printf("Connected, interface %d\n",itf);
+    if (background) {
+	pid_t pid;
+
+	pid = fork();
+	if (pid < 0) {
+	    perror("fork");
+	    return 1;
+	}
+	if (pid) return 0;
+    }
     while (1) {
 	fd_set set;
 	int ret;
