@@ -3,9 +3,6 @@
 /* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
  
 
-#if 1 /* debugging only */
-#include <stdio.h>
-#endif
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
@@ -92,11 +89,10 @@
 #define NORMALIZE(x,b) (((x)-(b)) & 0xffffff)
 #define NORM_RX(v) NORMALIZE((v),dsc->vr_r)
 #define NORM_TX(v) NORMALIZE((v),dsc->vt_a)
+#define NORM_TXP(v) NORMALIZE((v),dsc->vt_pa)
+#define NEGATIVE(v) ((v) & 0x800000)
 
 
-static const char *pdu_name[] = { "???","BGN","BGAK","END","ENDAK","RS","RSAK",
-  "BGREJ","SD","ER","POLL","STAT","USTAT","UD","MD","ERAK" };
-
 static const char *state_name[] = { "Idle","OutConnPend","InConnPend",
   "OutDiscPend","OutResyPend","InResyPend","OutRecPend","RecRespPend",
   "InRecPend","DataTransReady" };
@@ -145,8 +141,8 @@ static void maa_error(void *arg,char code,int count)
 
 static int sdu_length(BUFFER *buf)
 {
-     return buf->length-4-SSCOP_PAD(*(unsigned long *) ((char *) buf->data+
-       buf->length-4));
+    return buf->length-4-SSCOP_PAD(*(__u32 *) ((char *) buf->data+
+      buf->length-4));
 }
 
 
@@ -154,7 +150,7 @@ static BUFFER *build_pdu(SSCOP_DSC *dsc,unsigned char type,void *data,
   int num)
 {
     BUFFER *buf;
-    unsigned long *trailer;
+    __u32 *trailer;
     int pad;
 
     buf = buffer_create(num+12,dsc->vt_s); /* space for trailers */
@@ -162,11 +158,11 @@ static BUFFER *build_pdu(SSCOP_DSC *dsc,unsigned char type,void *data,
 	memset((char *) buf->data+(num & ~3),0,4); /* clear padding area */
 	memcpy((unsigned char *) buf->data,data,num);
 	pad = (4-(num & 3)) & 3;
-	trailer = (unsigned long *) ((char *) buf->data+num+pad);
+	trailer = (__u32 *) ((char *) buf->data+num+pad);
     }
     else {
 	pad = 0;
-	trailer = (unsigned long *) buf->data;
+	trailer = (__u32 *) buf->data;
     }
     diag(COMPONENT,DIAG_DEBUG,"generating %s PDU",pdu_name[type]);
     switch (type) {
@@ -218,24 +214,10 @@ static BUFFER *build_pdu(SSCOP_DSC *dsc,unsigned char type,void *data,
 }
 
 
-static void dump_pdu(void *data,int len)
-{
-    int i;
-
-    if (!debug) return;
-    for (i = 0; i < len; i++) {
-	if (!(i & 15)) printf("%s%04X:",i ? "\n" : "",i);
-	printf(" %02X",((unsigned char *) data)[i]);
-    }
-    putchar('\n');
-}
-
-
 static void emit_pdu(SSCOP_DSC *dsc,BUFFER *buf)
 {
     BUFFER **last;
 
-    dump_pdu(buf->data,buf->length);
     {
 	PDU_VARS;
 
@@ -245,7 +227,7 @@ static void emit_pdu(SSCOP_DSC *dsc,BUFFER *buf)
     }
     if (dsc->ops->cpcs_send)
 	dsc->ops->cpcs_send(dsc->user,buf->data,buf->length);
-    switch (SSCOP_TYPE(*(unsigned long *) ((char *) buf->data+buf->length-4))) {
+    switch (SSCOP_TYPE(*(__u32 *) ((char *) buf->data+buf->length-4))) {
 	case SSCOP_BGN:
 	    last = &dsc->last_bgn;
 	    break;
@@ -304,7 +286,7 @@ static void bad_pdu(SSCOP_DSC *dsc,unsigned char type)
 
 static void send_ustat(SSCOP_DSC *dsc,int first,int last)
 {
-    unsigned long range[2];
+    __u32 range[2];
 
     range[0] = htonl(first);
     range[1] = htonl(last);
@@ -728,7 +710,7 @@ static void data_poll(SSCOP_DSC *dsc,int s) /* 20.41-42 */
 {
     int curr,i;
 
-    if (!dsc->list) dsc->list = alloc(dsc->cf_max_stat*sizeof(int));
+    if (!dsc->list) dsc->list = alloc(dsc->cf_max_stat*sizeof(__u32));
     if (NORM_RX(dsc->vr_h) > NORM_RX(s)) {
 	start_error_recov(dsc,'Q');
 	return;
@@ -797,15 +779,14 @@ static void data_ustat(SSCOP_DSC *dsc,int mr,int r,int e1,int e2) /* 20.43 */
 }
 
 
-static void data_stat(SSCOP_DSC *dsc,int ps,int mr,int r,unsigned long *list,
+static void data_stat(SSCOP_DSC *dsc,int ps,int mr,int r,__u32 *list,
   int length) /* 20.44-46 */
 {
     BUFFER *buf,*buf2;
-    unsigned long *curr,tmp;
+    __u32 *curr,tmp;
     int i,count,seq1,seq2;
 
-    if (NORM_TX(dsc->vt_pa) > NORM_TX(ps) || NORM_TX(ps) >
-      NORM_TX(dsc->vt_ps)) {
+    if (NEGATIVE(NORM_TXP(ps)) || NORM_TXP(ps) > NORM_TXP(dsc->vt_ps)) {
 	start_error_recov(dsc,'R');
 	return;
     }
@@ -854,8 +835,8 @@ static void data_stat(SSCOP_DSC *dsc,int ps,int mr,int r,unsigned long *list,
 		    start_error_recov(dsc,'S');
 		    return;
 		}
-		if (NORM_TX(buf->extra) < NORM_TX(ps) &&
-		  NORM_TX(ps) <= NORM_TX(dsc->vt_ps) &&
+		if (NEGATIVE(NORM_TXP(buf->extra)) &&
+		  NORM_TXP(ps) <= NORM_TXP(dsc->vt_ps) &&
 		  !queue_lookup(&dsc->rt_q,seq1)) {
 		    buf2 = buffer_clone(buf);
 		    queue_put(&dsc->rt_q,buf2);
@@ -914,7 +895,6 @@ static int handle_sscop_pdu(SSCOP_DSC *dsc,void *msg,int size)
 {
     PDU_VARS;
 
-    dump_pdu(msg,size);
     if (DECOMPOSE_PDU(dsc,msg,size)) return 1;
     PRINT_PDU("RECV",msg);
     switch (type) {
@@ -1450,8 +1430,8 @@ static int handle_sscop_pdu(SSCOP_DSC *dsc,void *msg,int size)
 		    data_poll(dsc,s);
 		    return 1;
 		case SSCOP_USTAT:
-		    data_ustat(dsc,mr,r,SSCOP_N(((unsigned long *) msg)[0]),
-		      SSCOP_N(((unsigned long *) msg)[1]));
+		    data_ustat(dsc,mr,r,SSCOP_N(((__u32 *) msg)[0]),
+		      SSCOP_N(((__u32 *) msg)[1]));
 		    return 1;
 		case SSCOP_STAT:
 		    data_stat(dsc,ps,mr,r,msg,length/4);
