@@ -72,15 +72,19 @@ void from_net(void *buffer,int size)
 
 void to_signaling(void *msg,int size)
 {
-    trace_q2931("TO NETWORK",msg,size);
+    trace_uni("TO NETWORK",msg,size);
+    diag(COMPONENT,DIAG_DEBUG,"TO SAAL: %s (0x%02x) CR 0x%06x (%d bytes)",
+      mid2name(((unsigned char *) msg)[5]),((unsigned char *) msg)[5],
+      (((unsigned char *) msg)[2] << 16) | (((unsigned char *) msg)[3] << 8) |
+      ((unsigned char *) msg)[4],size);
     saal_send(&saal,msg,size);
 }
 
 
 static void q_data_ind(void *user_data,void *data,int length)
 {
-    trace_q2931("FROM NETWORK",data,length);
-    to_q2931(data,length);
+    trace_uni("FROM NETWORK",data,length);
+    to_uni(data,length);
 }
 
 
@@ -108,85 +112,38 @@ static SAAL_USER_OPS ops = {
 static volatile int got_usr1 = 0,got_usr2 = 0;
 
 
-static void dump_sap(FILE *file,const char *label,struct sockaddr_atmsvc *sap)
+static void dump_addr(FILE *file,const char *label,struct sockaddr_atmsvc *addr)
 {
-    struct atm_blli *blli;
-    int i,length;
+    char buffer[MAX_ATM_ADDR_LEN+1];
+    int i;
 
-    if (!sap) return;
+    if (!atmsvc_addr_in_use(*addr)) return;
     fprintf(file,"  %s ",label);
-    if (*sap->sas_addr.pub)
-	fprintf(file,"%s%s",sap->sas_addr.pub,*sap->sas_addr.prv ? "+":
-	  "\n    ");
-    if (*sap->sas_addr.prv) {
-	for (i = 0; i < ATM_ESA_LEN; i++)
-	    fprintf(file,"%02X",sap->sas_addr.prv[i]);
-	fprintf(file,"\n    ");
+    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) addr,A2T_NAME |
+      A2T_PRETTY | A2T_LOCAL) >= 0) fprintf(file,"%s\n",buffer);
+    else {
+	fprintf(file,"<invalid:");
+	for (i = 0; i < sizeof(*addr); i++)
+	    fprintf(file," %02X",((unsigned char *) addr)[i]);
+	fprintf(file,">\n");
     }
-    if (sap->sas_addr.bhli.hl_type) {
-	length =
-#ifdef UNI30
-	   sap->sas_addr.bhli.hl_type == ATM_HL_HLP ? 4 :
-#endif
-	  sap->sas_addr.bhli.hl_type == ATM_HL_VENDOR ? 7 :
-	  sap->sas_addr.bhli.hl_length;
-	fprintf(file,"BHLI %d[%d] = { ",sap->sas_addr.bhli.hl_type,length);
-	for (i = 0; i < length; i++)
-	    fprintf(file,"%s%02X",i ? "," : "",sap->sas_addr.bhli.hl_info[i]);
-	fprintf(file," }\n    ");
+}
+
+
+static void dump_sap(FILE *file,const char *label,struct atm_sap *sap)
+{
+    char buffer[MAX_ATM_SAP_LEN+1];
+    int i;
+
+    fprintf(file,"  %s ",label);
+    if (sap2text(buffer,MAX_ATM_SAP_LEN+1,sap,S2T_NAME | S2T_LOCAL) >= 0)
+	fprintf(file,"%s\n",buffer);
+    else {
+	fprintf(file,"<invalid:");
+	for (i = 0; i < sizeof(*sap); i++)
+	    fprintf(file," %02X",((unsigned char *) sap)[i]);
+	fprintf(file,">\n");
     }
-    for (blli = sap->sas_addr.blli; blli; blli = blli->next) {
-	fprintf(file,"BLLI");
-	if (blli->l2_proto) {
-	    fprintf(file,"  L2 %d",blli->l2_proto);
-	    switch (blli->l2_proto) {
-		case ATM_L2_X25_LL:
-		case ATM_L2_X25_ML:
-		case ATM_L2_HDLC_ARM:
-		case ATM_L2_HDLC_NRM:
-		case ATM_L2_HDLC_ABM:
-		case ATM_L2_Q922:
-		case ATM_L2_ISO7776:
-		    fprintf(file," mode=%d, window=%d\n    ",
-		      blli->l2.itu.mode,blli->l2.itu.window);
-		    break;
-		case ATM_L2_USER:
-		    fprintf(file," user=0x%x\n    ",blli->l2.user);
-		    break;
-		default:
-		    fprintf(file,"\n    ");
-	    }
-	}
-	if (blli->l3_proto) {
-	    fprintf(file,"  L3 %d",blli->l3_proto);
-	    switch (blli->l3_proto) {
-		case ATM_L3_X25:
-		case ATM_L3_ISO8208:
-		case ATM_L3_X223:
-		    fprintf(file," mode=%d, def=%d, pack=%d\n    ",
-		      blli->l3.itu.mode,blli->l3.itu.def_size,
-		      blli->l3.itu.window);
-		    break;
-		case ATM_L3_TR9577:
-		    fprintf(file," ipi=0x%x",blli->l3.tr9577.ipi);
-		    if (blli->l3.tr9577.ipi != NLPID_IEEE802_1_SNAP)
-			fprintf(file,"\n    ");
-		    else {
-			for (i = 0; i < 5; i++)
-			    fprintf(file,"%s%02X",i ? "," : " ",
-			      blli->l3.tr9577.snap[i]);
-			fprintf(file,"\n    ");
-		    }
-		    break;
-		case ATM_L3_USER:
-		    fprintf(file," user=0x%x\n    ",blli->l3.user);
-		    break;
-		default:
-		    fprintf(file,"\n    ");
-	    }
-	}
-    }
-    fprintf(file,"\n");
 }
 
 
@@ -199,8 +156,9 @@ static void dump_status(FILE *file,const char *banner)
 	fprintf(file,"0x%lx: %s, CR 0x%06lX, PVC %d.%d.%d\n",walk->id,
 	  state_name[walk->state],walk->call_ref,walk->pvc.sap_addr.itf,
 	  walk->pvc.sap_addr.vpi,walk->pvc.sap_addr.vci);
-        dump_sap(file,"local ",walk->local);
-        dump_sap(file,"remote",walk->remote);
+        dump_addr(file,"local ",&walk->local);
+        dump_addr(file,"remote",&walk->remote);
+        dump_sap(file,"sap",&walk->sap);
     }
 }
 
@@ -322,9 +280,10 @@ static void trace_on_exit(int status,void *dummy)
 
 static void usage(const char *name)
 {
-    fprintf(stderr,"usage: %s [ -b ] [ -c config_file ] [ -d ] [ -D dump_dir ]"
-      " [ -l logfile ] [ -n ] [ -N ] [ -q qos ] [ -t trace_length ] "
-      "[ [itf.]vpi.vci ]\n",name);
+    fprintf(stderr,"usage: %s [ -b ] [ -c config_file ] [ -d ] "
+      "[ -D dump_dir ]\n"
+      "  [ -l logfile ] [ -n ] [ -N ]  [ -q qos ] [ -t trace_length ]\n"
+      "  [ [itf.]vpi.vci [ input output ] ]\n",name);
     exit(1);
 }
 
@@ -396,12 +355,18 @@ int main(int argc,char **argv)
 	    default:
 		usage(argv[0]);
 	}
-    if (optind == argc-1) {
+    if (optind < argc) {
 	if (text2atm(argv[optind],(struct sockaddr *) &signaling_pvc,
 	  sizeof(signaling_pvc),T2A_PVC) < 0)
 	    diag(COMPONENT,DIAG_FATAL,"text2atm \"%s\": failed",argv[optind]);
+	optind++;
+	if (optind == argc-2) {
+	    open_input(argv[optind]);
+	    open_output(argv[optind+1]);
+	    optind += 2;
+	}
     }
-    else if (optind != argc) usage(argv[0]);
+    if (optind != argc) usage(argv[0]);
     diag(COMPONENT,DIAG_INFO,"Linux ATM signaling "
 #ifdef UNI30
       "UNI 3.0"
@@ -411,6 +376,12 @@ int main(int argc,char **argv)
 #ifdef ALLOW_UNI30
       "+3.0compat"
 #endif
+#endif
+#ifdef UNI40
+      "UNI 4.0"
+#ifdef Q2963_1
+      "+Q.2931.1"
+#endif
 #endif
       "/AAL5, version " VERSION);
     if (dump_dir)
