@@ -3,10 +3,12 @@
 /* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
  
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -17,7 +19,8 @@
 
 #include "atm.h"
 #include "atmd.h"
-#include "q2931.h"
+#include "uni.h"
+#include "pdu.h"
 
 #include "proto.h"
 #include "io.h"
@@ -31,7 +34,7 @@ struct timeval now;
 struct sockaddr_atmpvc signaling_pvc;
 
 
-static int kernel,signaling;
+static int kernel_in = -1,kernel_out,signaling;
 
 
 /* ----- kernel interface -------------------------------------------------- */
@@ -53,63 +56,66 @@ static int open_kernel(void)
 }
 
 
-#define KERNEL_BASE_LEN (sizeof(struct atmsvc_msg)-sizeof(struct atm_blli))
-#define KERNEL_BUFFER_SIZE (sizeof(struct atmsvc_msg)+(ATM_MAX_BLLI-1)* \
-  sizeof(struct atm_blli)+1)
+void open_input(const char *name)
+{
+    kernel_in = open(name,O_RDWR); /* actually read-only, but that'd block */
+    if (kernel_in >= 0) return;
+    perror(name);
+    exit(1);
+}
+
+
+void open_output(const char *name)
+{
+    kernel_out = open(name,O_RDWR); /* actually write-only */
+    if (kernel_out >= 0) return;
+    perror(name);
+    exit(1);
+}
 
 
 static void recv_kernel(void)
 {
-    static unsigned char buffer[KERNEL_BUFFER_SIZE];
-    int size,bllis;
+    static unsigned char buffer[sizeof(struct atmsvc_msg)+1];
+    int size;
 
-    size = read(kernel,buffer,KERNEL_BUFFER_SIZE);
+    size = read(kernel_in,buffer,sizeof(buffer));
     if (size < 0) {
 	diag(COMPONENT,DIAG_ERROR,"read kernel: %s",strerror(errno));
 	return;
     }
-    if (size < KERNEL_BASE_LEN)
-	diag(COMPONENT,DIAG_FATAL,"kernel message too short (%d < %d)",size,
-	  KERNEL_BASE_LEN);
-    if (size == KERNEL_BUFFER_SIZE) /* gotcha ! */
-	diag(COMPONENT,DIAG_FATAL,"kernel message too big (>= %d)",
-	  KERNEL_BUFFER_SIZE);
-    trace_kernel("FROM KERNEL",(struct atmsvc_msg *) buffer,size);
-    bllis = (size-KERNEL_BASE_LEN)/sizeof(struct atm_blli);
-    if (size != KERNEL_BASE_LEN+bllis*sizeof(struct atm_blli))
-	diag(COMPONENT,DIAG_FATAL,
-	  "kernel message has bad length (got %d, base is %d, incr is %d)",
-	  size,KERNEL_BASE_LEN,sizeof(struct atm_blli));
+    if (size != sizeof(struct atmsvc_msg))
+	diag(COMPONENT,DIAG_FATAL,"kernel message size %d != %d",size,
+	  sizeof(struct atmsvc_msg));
+    trace_kernel("FROM KERNEL",(struct atmsvc_msg *) buffer);
     from_kernel((struct atmsvc_msg *) buffer,size);
 }
 
 
 void to_kernel(struct atmsvc_msg *msg)
 {
-    struct atm_blli *walk;
-    int size,wrote;
+    int wrote;
 
-    size = bllis2msg(0);
-    for (walk = msg->svc.sas_addr.blli; walk; walk = walk->next)
-	size += sizeof(struct atm_blli);
-    diag("KERNEL",DIAG_DEBUG,"TO KERNEL: %s (%d) for 0x%lx/0x%lx <%d>",
-      as_name[msg->type],msg->reply,msg->vcc,msg->listen_vcc,size);
+    diag("KERNEL",DIAG_DEBUG,"TO KERNEL: %s (%d) for 0x%lx/0x%lx",
+      as_name[msg->type],msg->reply,msg->vcc,msg->listen_vcc);
 	/* should be "IO" ... */
-    trace_kernel("TO KERNEL",msg,size);
-    wrote = write(kernel,msg,size);
-    if (wrote == size) return;
+    trace_kernel("TO KERNEL",msg);
+    wrote = write(kernel_out,msg,sizeof(*msg));
+    if (wrote == sizeof(*msg)) return;
     if (wrote < 0) {
 	perror("kernel write");
 	return;
     }
-    diag(COMPONENT,DIAG_ERROR,"bad kernel write: wanted %d, wrote %d",size,
-      wrote);
+    diag(COMPONENT,DIAG_ERROR,"bad kernel write: wanted %d, wrote %d",
+      sizeof(*msg),wrote);
 }
 
 
 static void close_kernel(void)
 {
-    (void) close(kernel); /* may get major complaints from the kernel ... */
+    (void) close(kernel_in); /* may get major complaints from the kernel ... */
+    (void) close(kernel_out); /* fails if kernel_in == kernel_out, but that's
+				 okay */
 }
 
 
@@ -165,31 +171,21 @@ static void recv_signaling(void)
 	perror("read signaling");
 	return;
     }
-#if 0
-    {
-	int i;
-
-	for (i = 0; i < size; i++) printf("%02X ",buffer[i]);
-	if (size) putchar('\n');
-    }
-#endif
+    diag(COMPONENT,DIAG_DEBUG,"FROM NET: %s PDU (%d bytes)",
+      pdu_name[size > 3 && !(size & 3) ? buffer[size-4] & 0xf : 0],size);
+    if (debug) diag_dump(COMPONENT,DIAG_DEBUG,NULL,buffer,size);
     from_net(buffer,size);
 }
 
 
 void to_net(void *msg,int size)
 {
-    int wrote,i;
+    int wrote;
 
-    diag(COMPONENT,DIAG_DEBUG,"TO NET: %s (0x%02x) CR 0x%06x (%d bytes)",
-      mid2name(((unsigned char *) msg)[5]),((unsigned char *) msg)[5],
-      (((unsigned char *) msg)[2] << 16) |
-      (((unsigned char *) msg)[3] << 8) | ((unsigned char *) msg)[4],
-      size);
-    if (debug) {
-	for (i = 0; i < size; i++) printf("%02X ",((unsigned char *) msg)[i]);
-	if (size) putchar('\n');
-    }
+    diag(COMPONENT,DIAG_DEBUG,"TO NET: %s PDU (%d bytes)",
+      pdu_name[size > 3 && !(size & 3) ? ((unsigned char *) msg)[size-4] & 0xf
+      : 0],size);
+    if (debug) diag_dump(COMPONENT,DIAG_DEBUG,NULL,msg,size);
     wrote = write(signaling,msg,size);
     if (wrote == size) return;
     if (wrote < 0) {
@@ -228,7 +224,7 @@ int get_addr(int itf)
     req.number = itf;
     req.arg = buffer;
     req.length = sizeof(buffer);
-    if (ioctl(kernel,ATM_GETADDR,&req) < 0)
+    if (ioctl(signaling,ATM_GETADDR,&req) < 0)
 	diag(COMPONENT,DIAG_FATAL,"ioctl ATM_GETADDR yields \"%s\"",
 	  strerror(errno));
     addrs = req.length/sizeof(struct sockaddr_atmsvc);
@@ -255,8 +251,8 @@ int get_addr(int itf)
 
 int open_all(void)
 {
-    kernel = open_kernel();
-    if (kernel < 0) return -1;
+    if (kernel_in == -1) kernel_in = kernel_out = open_kernel();
+    if (kernel_in < 0) return -1;
     signaling = open_signaling();
     if (signaling < 0) {
 	close_kernel();
@@ -286,9 +282,9 @@ void poll_loop(void)
     int fds,ret;
 
     FD_ZERO(&perm);
-    FD_SET(kernel,&perm);
+    FD_SET(kernel_in,&perm);
     FD_SET(signaling,&perm);
-    fds = kernel > signaling ? kernel+1 : signaling+1;
+    fds = kernel_in > signaling ? kernel_in+1 : signaling+1;
     gettimeofday(&now,NULL);
     while (1) {
 	set = perm;
@@ -307,7 +303,7 @@ void poll_loop(void)
 	else {
 	    diag(COMPONENT,DIAG_DEBUG,"----------");
 	    gettimeofday(&now,NULL);
-	    if (FD_ISSET(kernel,&set)) recv_kernel();
+	    if (FD_ISSET(kernel_in,&set)) recv_kernel();
 	    if (FD_ISSET(signaling,&set)) recv_signaling();
 	    expire_timers();
 	      /* expire timers after handling messges to make sure we don't
