@@ -25,6 +25,8 @@ extern struct sockaddr_atmpvc signaling_pvc;
 
 
 int open_all(void);
+void open_input(const char *name);
+void open_output(const char *name);
 void init_current_time(void);
 void poll_loop(void);
 void close_all(void);
