@@ -15,11 +15,19 @@
 typedef enum { /* formatting aligned with state_map and others */
 	ss_invalid,	ss_null,	ss_listening,	ss_connecting,
 	ss_connected,	ss_indicated,	ss_accepting,	ss_zombie,
-	ss_wait_rel,	ss_wait_close,	ss_rel_req,	ss_rel_ind
+	ss_wait_rel,	ss_wait_close,	ss_rel_req,	ss_rel_ind,
+#ifdef Q2963_1
+	ss_mod_lcl,	ss_mod_req,	ss_mod_rcv,	ss_mod_fin_ack,
+	ss_mod_fin_ok,	ss_mod_fin_fail
+#endif
 } STATE;
 
-typedef enum { qs_null,qs_call_init,qs_out_proc = 3,qs_conn_req = 8,
-  qs_in_proc,qs_active,qs_rel_req,qs_rel_ind } Q2931_STATE;
+typedef enum { cs_null,cs_call_init,cs_out_proc = 3,cs_conn_req = 8,
+  cs_in_proc,cs_active,cs_rel_req,cs_rel_ind,
+#ifdef Q2963_1
+  cs_mod_req,cs_mod_rcv
+#endif
+  } CALL_STATE;
 
 typedef enum { ps_null,ps_add_init,ps_add_recv = 6,ps_drop_init = 11,
   ps_drop_recv,ps_active = 10 } PARTY_STATE;
@@ -29,12 +37,16 @@ typedef struct _socket {
     struct sockaddr_atmpvc pvc;
     /* --- socket layer information ---------------------------------------- */
     unsigned long id;
-    struct sockaddr_atmsvc *local; /* local SAP */
-    struct sockaddr_atmsvc *remote; /* remote SAP */
+    struct sockaddr_atmsvc local; /* local address */
+    struct sockaddr_atmsvc remote; /* remote address */
+    struct atm_sap sap; /* SAP (BHLI and BLLI) */
     struct atm_qos qos; /* QOS parameters */
+#ifdef Q2963_1
+    struct atm_qos new_qos; /* during modification */
+#endif
     int error; /* error code for close */
-    /* --- Q.93B/Q.2931 information ---------------------------------------- */
-    Q2931_STATE q2931_state;
+    /* --- UNI information ------------------------------------------------- */
+    CALL_STATE call_state;
     unsigned long call_ref; /* bit 24 like when sending */
     short ep_ref; /* endpoint reference; -1 for p2p */
     TIMER *conn_timer; /* current connection timer */
@@ -51,9 +63,9 @@ typedef struct _socket {
  */
 
 
-extern const Q2931_STATE state_map[];
+extern const CALL_STATE state_map[];
 extern const PARTY_STATE eps_map[];
-extern const char *state_name[],*qs_name[],*as_name[];
+extern const char *state_name[],*cs_name[],*as_name[];
 
 extern SOCKET *sockets;
 extern unsigned char q_buffer[];
@@ -67,12 +79,8 @@ extern const char *sig_qos;
 extern const char *dump_dir;
 
 
-#define msg2bllis(s) (((int) (s)-(int) sizeof(struct atmsvc_msg))/ \
-  (int) sizeof(struct atm_blli)+1)
-#define bllis2msg(n) (sizeof(struct atmsvc_msg)+sizeof(struct atm_blli)*((n)-1))
-
 #define SEND_ERROR(vcc,code) \
-  send_kernel(vcc,0L,as_error,code,NULL,NULL,NULL,NULL)
+  send_kernel(vcc,0L,as_error,code,NULL,NULL,NULL,NULL,NULL)
 
 
 void poll_signals(void);
@@ -80,11 +88,11 @@ void poll_signals(void);
 void from_kernel(struct atmsvc_msg *msg,int size);
 void itf_load(int itf);
 
-void to_q2931(void *msg,int size);
+void to_uni(void *msg,int size);
 void send_kernel(unsigned long vcc,unsigned long listen_vcc,
   enum atmsvc_msg_type type,int reply,const struct sockaddr_atmpvc *pvc,
   const struct sockaddr_atmsvc *svc,const struct sockaddr_atmsvc *local,
-  const struct atm_qos *qos);
+  const struct atm_sap *sap,const struct atm_qos *qos);
 void from_net(void *msg,int size);
 void to_signaling(void *msg,int size);
 void saal_failure(void);
@@ -94,10 +102,14 @@ void clear_all_calls_on_T309(void);
 SOCKET *new_sock(unsigned long id);
 void free_sock(SOCKET *sock);
 void new_state(SOCKET *sock,STATE state);
-SOCKET *lookup_sap(const struct sockaddr_atmsvc *sap,const struct atm_qos *qos,
-  struct sockaddr_atmsvc **res_sap,struct atm_qos *res_qos);
+SOCKET *lookup_sap(const struct sockaddr_atmsvc *addr,
+  const struct atm_sap *sap,const struct atm_qos *qos,
+  struct sockaddr_atmsvc *res_addr,struct atm_sap *res_sap,
+  struct atm_qos *res_qos,int exact_match);
+
 void send_release(SOCKET *sock,unsigned char reason,...);
 void send_release_complete(unsigned long call_ref,unsigned char cause);
+void send_modify_reject(SOCKET *sock,unsigned char reason);
 
 const char *mid2name(unsigned char mid);
 
