@@ -1,20 +1,21 @@
-/* timeout.c - Processing of Q.2931 timeout events */
+/* timeout.c - Processing of signaling timeout events */
  
-/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdio.h>
 #include <errno.h>
 #include <assert.h>
+#include <linux/atmdev.h>
 
 #include "atmd.h"
 
-#include "q2931.h"
+#include "uni.h"
 #include "proto.h"
 #include "timeout.h"
 
 
-#define COMPONENT "Q2931"
+#define COMPONENT "UNI"
 
 
 static void complain(SOCKET *sock,const char *timer)
@@ -26,9 +27,8 @@ static void complain(SOCKET *sock,const char *timer)
 
 void on_T303(void *user) /* CONNECTING */
 {
-    SOCKET *sock;
+    SOCKET *sock = user;
 
-    sock = user;
     diag(COMPONENT,DIAG_DEBUG,"T303 on 0x%lx",sock->id);
     if (sock->state != ss_connecting) complain(sock,"T303");
     SEND_ERROR(sock->id,-ETIMEDOUT);
@@ -39,9 +39,8 @@ void on_T303(void *user) /* CONNECTING */
 
 void on_T308_1(void *user) /* WAIT_REL or REL_REQ */
 {
-    SOCKET *sock;
+    SOCKET *sock = user;
 
-    sock = user;
     diag(COMPONENT,DIAG_DEBUG,"T308_1 on 0x%lx",sock->id);
     if (sock->state != ss_wait_rel && sock->state != ss_rel_req)
 	complain(sock,"T308_1");
@@ -53,9 +52,8 @@ void on_T308_1(void *user) /* WAIT_REL or REL_REQ */
 
 void on_T308_2(void *user) /* WAIT_REL or REL_REQ */
 {
-    SOCKET *sock;
+    SOCKET *sock = user;
 
-    sock = user;
     diag(COMPONENT,DIAG_WARN,"Trouble: T308_2 has expired");
     if (sock->state != ss_wait_rel && sock->state != ss_rel_req)
 	complain(sock,"T308_2");
@@ -82,9 +80,8 @@ void on_T310(void *user)
 
 void on_T313(void *user) /* ACCEPTING */
 {
-    SOCKET *sock;
+    SOCKET *sock = user;
 
-    sock = user;
     diag(COMPONENT,DIAG_DEBUG,"T313 on 0x%lx",sock->id);
     if (sock->state != ss_accepting) complain(sock,"T313");
     send_release(sock,ATM_CV_TIMER_EXP,313);
@@ -92,3 +89,33 @@ void on_T313(void *user) /* ACCEPTING */
     START_TIMER(sock,T308_1);
     new_state(sock,ss_rel_req);
 }
+
+
+#ifdef Q2963_1
+
+void on_T360(void *user)
+{
+    SOCKET *sock = user;
+
+    diag(COMPONENT,DIAG_DEBUG,"T360 on 0x%lx",sock->id);
+    if (sock->state != ss_mod_req) complain(sock,"T360");
+    send_release(sock,ATM_CV_TIMER_EXP,360);
+    sock->conn_timer = NULL;
+    START_TIMER(sock,T308_1);
+    new_state(sock,ss_rel_req);
+}
+
+
+void on_T361(void *user)
+{
+    SOCKET *sock = user;
+
+    diag(COMPONENT,DIAG_DEBUG,"T361 on 0x%lx",sock->id);
+    if (sock->state != ss_connected) complain(sock,"T361");
+    sock->qos = sock->new_qos;
+    send_kernel(sock->id,0,as_modify,ATM_MF_SET,NULL,NULL,NULL,NULL,&sock->qos);
+    sock->conn_timer = NULL;
+    new_state(sock,ss_mod_fin_ack);
+}
+
+#endif
