@@ -1,4 +1,4 @@
-/* read.c - receive AAL5 PDU */
+/* aread.c - receive AAL5 PDU */
 
 /* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
 
@@ -19,25 +19,32 @@
 
 static void usage(const char *name)
 {
-    fprintf(stderr,"usage: %s [itf.]vpi.vci\n",name);
+    fprintf(stderr,"usage: %s [-c] [itf.]vpi.vci\n",name);
     exit(1);
 }
 
 
 int main(int argc,char **argv)
 {
+    const char *name;
     struct sockaddr_atmpvc addr;
     struct atm_qos qos;
-    int s;
+    int chars,s;
 
-    if (argc != 2) usage(argv[0]);
+    name = argv[0];
+    chars = argc == 3 && !strcmp(argv[1],"-c");
+    if (chars) {
+	argc--;
+	argv++;
+    }
+    if (argc != 2) usage(name);
     if ((s = socket(PF_ATMPVC,SOCK_DGRAM,0)) < 0) {
 	perror("socket");
 	return 1;
     }
     memset(&addr,0,sizeof(addr));
     if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),
-      T2A_PVC | T2A_UNSPEC | T2A_WILDCARD) < 0) usage(argv[0]);
+      T2A_PVC | T2A_UNSPEC | T2A_WILDCARD) < 0) usage(name);
     memset(&qos,0,sizeof(qos));
     qos.aal = ATM_AAL5;
     qos.rxtp.traffic_class = ATM_UBR;
@@ -60,7 +67,11 @@ int main(int argc,char **argv)
 	printf("%d",size);
 	if (size < 0) printf(" (%s)",strerror(errno));
 	printf(": ");
-	for (i = 0; i < size; i++) printf(" %02X",start[i]);
+	for (i = 0; i < size; i++)
+	    if (chars)
+		if (start[i] > ' ' && start[i] < 127) printf(" %c",start[i]);
+		else printf(" \\%03o",start[i]);
+	    else printf(" %02X",start[i]);
 	putchar('\n');
     }
 }
