@@ -60,6 +60,7 @@ static char RCSid[] = "ttcp.c $Revision: 1.12 $";
 #include <sys/time.h>		/* struct timeval */
 #include <linux/atm.h>
 #include <atm.h>
+#include <atmsap.h>
 #include <arequipa.h>
 
 #if defined(SYSV)
@@ -391,41 +392,53 @@ int no_check = 0;
         udp?SOCK_DGRAM:SOCK_STREAM,0)) < 0) err("socket");
     
     mes("socket");
-	if (arequipa && trans) {
-	    if (arequipa_preset(fd,&satm,&qos) < 0)
-		err("arequipa_preset");
-	}
-	if (atm) {
-	    if (setsockopt(fd,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
-		err("setsockopt SO_ATMQOS");
-	    }
-	}
-	else if (setsockopt(fd, SOL_SOCKET, SO_NO_CHECK, &no_check, sizeof(long)) <
-		0) err("setsockopt: no_check");
+    if (arequipa && trans) {
+	if (arequipa_preset(fd,&satm,&qos) < 0)
+	    err("arequipa_preset");
+    }
+    if (atm) {
+	if (setsockopt(fd,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0)
+	    err("setsockopt SO_ATMQOS");
+    }
+    else if (setsockopt(fd, SOL_SOCKET, SO_NO_CHECK, &no_check, sizeof(long)) <
+	    0) err("setsockopt: no_check");
+#ifdef SO_ATMSAP
+    if (atm && satm.sas_family == AF_ATMSVC) {
+	struct atm_sap sap;
+
+	memset(&sap,0,sizeof(sap));
+	sap.bhli.hl_type = ATM_HL_VENDOR;
+	sap.bhli.hl_length = 7;
+	memcpy(sap.bhli.hl_info,TTCP_HLT_VS_ID,7);
+	if (setsockopt(fd,SOL_ATM,SO_ATMSAP,&sap,sizeof(sap)) < 0)
+	    err("setsockopt SO_ATMSAP");
+    }
+#endif
     /* set socket buffer size */
 #if defined(SO_SNDBUF) || defined(SO_RCVBUF)
     if (sockbufsize) {
-int l;
+	int len;
+
 	if (trans) {
 	    /* set send socket buffer if we are transmitting */    
 	    if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sockbufsize,
 			   sizeof sockbufsize) < 0)
 		err("setsockopt: sndbuf");
-l = sizeof sockbufsize;
-if (getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sockbufsize, &l) < 0)
-perror("grr");
+	    len = sizeof sockbufsize;
+	    if (getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &sockbufsize, &len) < 0)
+		perror("getsockopt: sndbuf");
 	    mes("sndbuf");
 	} else {
 	    /* set receive socket buffer if we are receiving */    
 	    if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &sockbufsize,
 			   sizeof sockbufsize) < 0)
 		err("setsockopt: rcvbuf");
-l = sizeof sockbufsize;
-if (getsockopt(fd, SOL_SOCKET, SO_RCVBUF, &sockbufsize, &l) < 0)
-perror("grr");
+	    len = sizeof sockbufsize;
+	    if (getsockopt(fd, SOL_SOCKET, SO_RCVBUF, &sockbufsize, &len) < 0)
+		perror("getsockopt: rcvbuf");
 	    mes("rcvbuf");
 	}
-printf("sbs=%d\n",sockbufsize);
+	printf("real buffer size = %d\n",sockbufsize);
     }
 #endif
 
