@@ -65,7 +65,7 @@ void main(int argc, char **argv) {
    struct atm_qos qos1, qos2;
    char *buffer;
    char buffer2[REPLY_SIZE];
-   int blocks, s1, s2;
+   int s1, s2;
    size_t size;
 
    char *theaddr1, *theaddr2;
@@ -74,7 +74,7 @@ void main(int argc, char **argv) {
    int WINDOW_SIZE;
 
   
-   int i, w, j;
+   int i, w;
    int sending = 0;
    double t1, t2;
 
